@@ -11,6 +11,9 @@ from . DNN import DNN
 from . DNN import DNN5
 from . DNN import DNNordinal
 from . DNN import DNNdomain
+from . DNN import DNNregion
+from . util_Mouse import report_region
+
 from . TrainerExe import TrainerExe
 import pickle
 
@@ -24,7 +27,7 @@ def seed_worker(worker_id):
     np.random.seed(worker_seed)
     random.seed(worker_seed)
 
-def Fit_cord (data_train, location_data = None, hidden_dims = [30, 25, 15], num_epochs_max = 500, path = "", filename = "PreOrg_Mousesc", batch_size = 4, num_workers = 4, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
+def Fit_cord (data_train, location_data = None, hidden_dims = [30, 25, 15], num_epochs_max = 500, path = "", filename = "PreOrg_Mousesc", batch_size = 4, num_workers = 1, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
     #
     random.seed(seednum)
     torch.manual_seed(seednum)
@@ -59,7 +62,40 @@ def Fit_cord (data_train, location_data = None, hidden_dims = [30, 25, 15], num_
     pickle.dump(DNNmodel, filehandler2)
     return DNNmodel
 
-def Fit_layer (data_train, layer_weights, layer_data = None, layerkey = "layer", hidden_dims = [10, 5, 2], num_epochs_max = 500, path = "", filename = "PreOrg_layersc", batch_size = 4, num_workers = 4, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
+
+def Fit_region (data_train, alpha = 0.95, location_data = None, hidden_dims = [30, 25, 15], num_epochs_max = 500, path = "", filename = "PreOrg_Mousesc", batch_size = 4, num_workers = 1, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
+    #
+    random.seed(seednum)
+    torch.manual_seed(seednum)
+    np.random.seed(seednum)
+    g = torch.Generator()
+    g.manual_seed(seednum)
+    #
+    if location_data is None:
+        location_data = data_train.obs
+    #
+    traindata = (data_train.X.A if issparse(data_train.X) else data_train.X)
+    tdatax = np.expand_dims(traindata, axis = 0)
+    tdata_rs = np.swapaxes(tdatax, 1, 2)
+    DataTra = wrap_gene_location(tdata_rs, location_data)
+    t_loader= torch.utils.data.DataLoader(DataTra, batch_size=batch_size, num_workers = num_workers, shuffle = True, worker_init_fn=seed_worker, generator=g)
+    # Create Deep Neural Network for Coordinate Regression
+    DNNmodel = DNNregion( in_channels = DataTra[1][0].shape[0], alpha = alpha,  hidden_dims = hidden_dims) # [100,50,25] )
+    DNNmodel = DNNmodel.float()
+    #
+    CoOrg = TrainerExe()
+    CoOrg.train(model = DNNmodel, train_loader = t_loader, num_epochs= num_epochs_max, RCcountMax = number_error_try, learning_rate = initial_learning_rate)
+    #
+    try:
+        os.makedirs("{path}".format(path = path))
+    except FileExistsError:
+        print("Folder already exists")
+    filename3 = "{path}/{filename}.obj".format(path = path, filename = filename) #"../output/CeLEry/Mousesc/PreOrg_Mousesc.obj"
+    filehandler2 = open(filename3, 'wb') 
+    pickle.dump(DNNmodel, filehandler2)
+    return DNNmodel    
+
+def Fit_layer (data_train, layer_weights, layer_data = None, layerkey = "layer", hidden_dims = [10, 5, 2], num_epochs_max = 500, path = "", filename = "PreOrg_layersc", batch_size = 4, num_workers = 1, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
     #
     random.seed(seednum)
     torch.manual_seed(seednum)
@@ -70,6 +106,7 @@ def Fit_layer (data_train, layer_weights, layer_data = None, layerkey = "layer",
     if layer_data is None:
         layer_data = data_train.obs
     #
+    layer_weights = torch.tensor(layer_weights.to_numpy())
     traindata = (data_train.X.A if issparse(data_train.X) else data_train.X)
     tdatax = np.expand_dims(traindata, axis = 0)
     tdata_rs = np.swapaxes(tdatax, 1, 2)
@@ -86,7 +123,7 @@ def Fit_layer (data_train, layer_weights, layer_data = None, layerkey = "layer",
     filehandler2 = open(filename3, 'wb') 
     pickle.dump(DNNmodel, filehandler2)
 
-def Fit_domain (data_train, domain_weights, domain_data = None, domainkey = "layer", hidden_dims = [10, 5, 2], num_epochs_max = 500, path = "", filename = "PreOrg_domainsc", batch_size = 4, num_workers = 4, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
+def Fit_domain (data_train, domain_weights, domain_data = None, domainkey = "layer", hidden_dims = [10, 5, 2], num_epochs_max = 500, path = "", filename = "PreOrg_domainsc", batch_size = 4, num_workers = 1, number_error_try = 15, initial_learning_rate = 0.0001, seednum = 2021):
     #
     random.seed(seednum)
     torch.manual_seed(seednum)
@@ -97,6 +134,7 @@ def Fit_domain (data_train, domain_weights, domain_data = None, domainkey = "lay
     if domain_data is None:
         domain_data = data_train.obs
     #
+    domain_weights = torch.tensor(domain_weights.to_numpy(), dtype=torch.float32)
     traindata = (data_train.X.A if issparse(data_train.X) else data_train.X)
     tdatax = np.expand_dims(traindata, axis = 0)
     tdata_rs = np.swapaxes(tdatax, 1, 2)
@@ -121,11 +159,13 @@ def Predict_cord (data_test, path = "", filename = "PreOrg_Mousesc", location_da
     vdatax = np.expand_dims(testdata, axis = 0)
     vdata_rs = np.swapaxes(vdatax, 1, 2)
     DataVal = wrap_gene_location(vdata_rs, location_data)
-    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 4)
+    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 1)
     #
     cord = report_prop_method_sc(folder = path,
                         name = filename, data_test = data_test,
                         Val_loader = Val_loader)
+    data_test.obs["x_cord_pred"] = cord[:,0]
+    data_test.obs["y_cord_pred"] = cord[:,1]
     return cord
 
 
@@ -150,6 +190,26 @@ def report_prop_method_sc (folder, name, data_test, Val_loader, outname = ""):
     np.savetxt("{folder}/{name}_predmatrix.csv".format(folder = folder, name = name), coords_predict, delimiter=",")
     return coords_predict
 
+
+def Predict_region (data_test, path = "", filename = "PreOrg_ConfScore", location_data = None, hist = ""):
+    testdata= (data_test.X.A if issparse(data_test.X) else data_test.X)
+    if location_data is None:
+        location_data = pd.DataFrame(np.ones((data_test.shape[0],2)), columns = ["psudo1", "psudo2"])
+    ## Wrap up Validation data in to dataloader
+    vdatax = np.expand_dims(testdata, axis = 0)
+    vdata_rs = np.swapaxes(vdatax, 1, 2)
+    DataVal = wrap_gene_location(vdata_rs, location_data)
+    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 1)
+    #
+    area_record = report_region(folder = path,
+                        name = filename, data_test = data_test,
+                        Val_loader = Val_loader, hist = hist)
+    conf_score = [1-i for i in area_record]
+    data_test.obs["area_record"] = np.array(area_record)
+    data_test.obs["conf_score"] = np.array(conf_score)
+    return conf_score
+
+
 def Predict_domain (data_test, class_num,  path = "", filename = "PreOrg_domainsc", truth_label = None, predtype = "probabilistic"):
     if truth_label is None:
         truth_label = "psudo_label"
@@ -159,13 +219,14 @@ def Predict_domain (data_test, class_num,  path = "", filename = "PreOrg_domains
     vdatax = np.expand_dims(testdata, axis = 0)
     vdata_rs = np.swapaxes(vdatax, 1, 2)
     DataVal = wrap_gene_domain(vdata_rs, location_data, truth_label)
-    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 4)
+    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 1)
     #
     domain = report_prop_method_domain(folder = path,
                        name = filename,
                        data_test = data_test,
                        Val_loader = Val_loader,
                        class_num = class_num)
+    data_test.obs["domain_cel_pred"] = domain[1]
     if predtype == "probabilistic":
         return domain[0]
     elif predtype == "deterministic":
@@ -251,13 +312,14 @@ def Predict_layer (data_test, class_num,  path = "", filename = "PreOrg_layernsc
     vdatax = np.expand_dims(testdata, axis = 0)
     vdata_rs = np.swapaxes(vdatax, 1, 2)
     DataVal = wrap_gene_layer(vdata_rs, location_data, layerkey = truth_label)
-    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 4)
+    Val_loader= torch.utils.data.DataLoader(DataVal, batch_size=1, num_workers = 1)
     #
     domain = report_prop_method_layer(folder = path,
                        name = filename,
                        data_test = data_test,
                        Val_loader = Val_loader,
                        class_num = class_num)
+    data_test.obs["layer_cel_pred"] = domain[1]
     if predtype == "probabilistic":
         return domain[0]
     elif predtype == "deterministic":
